@@ -6,6 +6,8 @@ use std::sync::Arc;
 use reqwest::redirect;
 use serde::{Deserialize, Serialize};
 
+use crate::res::OrdersInfo;
+
 macro_rules! get_client {
     ($self:expr) => {{
         #[cfg(feature = "single-client")]
@@ -19,6 +21,8 @@ macro_rules! get_client {
     }};
 }
 
+pub static SUCCESS: &str = "SUCCESS";
+
 #[derive(Debug, thiserror::Error)]
 pub enum Error {
     #[error("Client is not authorized. No bearer token available")]
@@ -39,10 +43,118 @@ pub enum Error {
     Refund,
     #[error("Create order returned invalid response")]
     CreateOrder,
+    #[error("Failed to fetch order transactions")]
+    OrderTransactions,
+    #[error("Failed to fetch order details")]
+    OrderDetails,
+    #[error("PayU rejected to create order with status {status_code:?}")]
+    CreateFailed {
+        status_code: String,
+        status_desc: Option<String>,
+        code: Option<String>,
+        severity: Option<String>,
+        code_literal: Option<CodeLiteral>,
+    },
+    #[error("PayU rejected to perform refund with status {status_code:?}")]
+    RefundFailed {
+        status_code: String,
+        status_desc: Option<String>,
+        code: Option<String>,
+        severity: Option<String>,
+        code_literal: Option<CodeLiteral>,
+    },
+    #[error("PayU rejected order details request with status {status_code:?}")]
+    OrderDetailsFailed {
+        status_code: String,
+        status_desc: Option<String>,
+        code: Option<String>,
+        severity: Option<String>,
+        code_literal: Option<CodeLiteral>,
+    },
+    #[error("PayU rejected order transactions details request with status {status_code:?}")]
+    OrderTransactionsFailed {
+        status_code: String,
+        status_desc: Option<String>,
+        code: Option<String>,
+        severity: Option<String>,
+        code_literal: Option<CodeLiteral>,
+    },
+    #[error("PayU returned order details but without any order")]
+    NoOrderInDetails,
 }
 
 pub type Result<T> = std::result::Result<T, Error>;
 
+/// PayU internal order id
+#[derive(
+    Debug,
+    Clone,
+    serde::Deserialize,
+    serde::Serialize,
+    derive_more::Display,
+    derive_more::From,
+    derive_more::Deref,
+)]
+#[serde(transparent)]
+pub struct OrderId(pub String);
+
+impl OrderId {
+    pub fn new<S: Into<String>>(id: S) -> Self {
+        Self(id.into())
+    }
+}
+
+/// PayU internal order id
+#[derive(
+    Debug,
+    serde::Deserialize,
+    serde::Serialize,
+    Copy,
+    Clone,
+    derive_more::Display,
+    derive_more::From,
+    derive_more::Deref,
+    derive_more::Constructor,
+)]
+#[serde(transparent)]
+pub struct MerchantPosId(pub i32);
+
+#[derive(
+    Debug,
+    Clone,
+    serde::Deserialize,
+    serde::Serialize,
+    derive_more::Display,
+    derive_more::From,
+    derive_more::Deref,
+)]
+#[serde(transparent)]
+pub struct ClientId(pub String);
+
+impl ClientId {
+    pub fn new<S: Into<String>>(id: S) -> Self {
+        Self(id.into())
+    }
+}
+
+#[derive(
+    Debug,
+    Clone,
+    serde::Deserialize,
+    serde::Serialize,
+    derive_more::Display,
+    derive_more::From,
+    derive_more::Deref,
+)]
+#[serde(transparent)]
+pub struct ClientSecret(pub String);
+
+impl ClientSecret {
+    pub fn new<S: Into<String>>(id: S) -> Self {
+        Self(id.into())
+    }
+}
+
 /// PayU payment status.
 ///
 /// Each payment is initially Pending and can change according to following
@@ -175,7 +287,6 @@ impl Buyer {
 
 pub type Price = i32;
 pub type Quantity = u32;
-pub type MerchantPosId = i32;
 
 #[derive(Serialize, Deserialize, Debug)]
 #[serde(rename_all = "camelCase")]
@@ -243,8 +354,8 @@ pub struct OrderCreateRequest {
     customer_ip: String,
     /// Secret pos ip. This is connected to PayU account
     #[serde(
-        serialize_with = "serialize::serialize_i32",
-        deserialize_with = "deserialize::deserialize_i32"
+        serialize_with = "serialize::serialize_newtype",
+        deserialize_with = "deserialize::deserialize_i32_newtype"
     )]
     merchant_pos_id: MerchantPosId,
     /// Transaction description
@@ -279,7 +390,7 @@ impl OrderCreateRequest {
         Self {
             notify_url: None,
             customer_ip: customer_ip.into(),
-            merchant_pos_id: 0,
+            merchant_pos_id: 0.into(),
             description: String::from(""),
             currency_code: currency.into(),
             total_amount: 0,
@@ -507,7 +618,7 @@ impl Status {
     /// assert_eq!(status.is_success(), true);
     /// ```
     pub fn is_success(&self) -> bool {
-        self.status_code.as_str() == "SUCCESS"
+        self.status_code.as_str() == SUCCESS
     }
 
     /// Returns http status
@@ -530,34 +641,161 @@ impl Status {
 
 #[derive(Deserialize, Debug)]
 #[serde(rename_all = "camelCase")]
-pub struct CreateOrderResult {
-    /// Http status as a text
-    pub status: Status,
-    /// Client should be redirected to this URI
-    pub redirect_uri: String,
-    /// This should be connected to your own order
-    pub order_id: String,
-    /// This is YOUR_EXT_ORDER_ID
-    pub ext_order_id: Option<String>,
+pub struct Prop {
+    pub name: String,
+    pub value: String,
 }
 
-#[derive(Deserialize, Debug)]
-#[serde(rename_all = "camelCase")]
-pub struct PartialRefundResult {
-    pub order_id: Option<String>,
-    pub refund: Option<Refund>,
-    pub status: Status,
+pub mod res {
+    use crate::{OrderId, Refund, Status};
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct CreateOrder {
+        /// Http status as a text
+        pub status: Status,
+        /// Client should be redirected to this URI
+        pub redirect_uri: String,
+        /// This should be connected to your own order
+        pub order_id: OrderId,
+        /// This is YOUR_EXT_ORDER_ID
+        pub ext_order_id: Option<String>,
+    }
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct PartialRefund {
+        pub order_id: Option<String>,
+        pub refund: Option<Refund>,
+        pub status: Status,
+    }
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct TransactionPayMethod {
+        pub value: String,
+    }
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "SCREAMING_SNAKE_CASE")]
+    pub enum CardProfile {
+        Consumer,
+        Business,
+    }
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "SCREAMING_SNAKE_CASE")]
+    pub enum CardClassification {
+        Debit,
+        Credit,
+    }
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct TransactionCartData {
+        /// // "543402******4014",
+        pub card_number_masked: String,
+        /// MC (MasterCard/Maestro), VS (Visa)
+        /// Example; "MC"
+        pub card_scheme: String,
+        pub card_profile: CardProfile,
+        pub card_classification: CardClassification,
+        ///  Example: "000"
+        pub card_response_code: String,
+        ///  Example: "000 - OK"
+        pub card_response_code_desc: String,
+        ///  Example: "5"
+        pub card_eci_code: String,
+        ///  Example: "AY",
+        pub card3ds_status: String,
+        ///  Example: "PL",
+        pub card_bin_country: String,
+        ///  Example: "MCC0111LL1121"
+        pub first_transaction_id: String,
+    }
+
+    /// > Installment proposal on the Sandbox environment is not related to the
+    /// > order amount and always returns data for 480 PLN.
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct TransactionCardInstallmentProposal {
+        /// Example: "5aff3ba8-0c37-4da1-ba4a-4ff24bcc2eed"
+        pub proposal_id: String,
+    }
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct TransactionCart {
+        pub cart_data: TransactionCartData,
+        pub card_installment_proposal: TransactionCardInstallmentProposal,
+    }
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct Transaction {
+        pub pay_method: TransactionPayMethod,
+        pub payment_flow: String,
+    }
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct Transactions {
+        pub transactions: Vec<Transaction>,
+    }
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct Order {
+        /// Example: "{orderId}",
+        pub order_id: super::OrderId,
+        /// Example:  "358766",
+        pub ext_order_id: Option<String>,
+        /// Example: "2014-10-27T14:58:17.443+01:00",
+        pub order_create_date: String,
+        /// Example: "http://localhost/OrderNotify/",
+        pub notify_url: Option<String>,
+        /// Example: "127.0.0.1",
+        pub customer_ip: String,
+        /// Example: "145227",
+        pub merchant_pos_id: String,
+        /// Example: "New order",
+        pub description: String,
+        /// Example: "PLN",
+        pub currency_code: String,
+        /// Example: "3200",
+        pub total_amount: String,
+        /// Example: "NEW",
+        pub status: String,
+        /// Example: `[{"name":"Product1","unitPrice":"1000","quantity":"1"}]`
+        pub products: Vec<super::Product>,
+    }
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct OrdersInfo {
+        pub orders: Vec<Order>,
+        pub status: super::Status,
+        pub properties: Option<Vec<crate::Prop>>,
+    }
+
+    #[derive(serde::Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct OrderInfo {
+        pub order: Order,
+        pub status: super::Status,
+        pub properties: Option<Vec<crate::Prop>>,
+    }
 }
 
 #[derive(Deserialize, Serialize, Debug)]
 #[serde(rename_all = "camelCase")]
 pub struct RefundRequest {
     description: String,
-    amount: Price,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    amount: Option<Price>,
 }
 
 impl RefundRequest {
-    pub fn new<Description>(description: Description, amount: Price) -> Self
+    pub fn new<Description>(description: Description, amount: Option<Price>) -> Self
     where
         Description: Into<String>,
     {
@@ -570,7 +808,8 @@ impl RefundRequest {
     pub fn description(&self) -> &str {
         &self.description
     }
-    pub fn amount(&self) -> Price {
+
+    pub fn amount(&self) -> Option<Price> {
         self.amount
     }
 }
@@ -591,13 +830,16 @@ pub struct Refund {
 pub mod notify {
     use serde::Deserialize;
 
+    use super::deserialize;
+    use crate::OrderId;
+
     /// Payment notification object received on [super::Order].[notify_url]
     #[derive(Deserialize, Debug)]
     #[serde(rename_all = "camelCase")]
     pub struct StatusUpdate {
         pub order: Order,
         pub local_receipt_date_time: Option<String>,
-        pub properties: Option<Vec<Prop>>,
+        pub properties: Option<Vec<super::Prop>>,
         pub status: Option<super::Status>,
     }
 
@@ -612,7 +854,7 @@ pub mod notify {
     #[serde(rename_all = "camelCase")]
     pub struct RefundUpdate {
         pub ext_order_id: String,
-        pub order_id: String,
+        pub order_id: OrderId,
         pub refund: Refund,
     }
 
@@ -632,33 +874,37 @@ pub mod notify {
     #[derive(Deserialize, Debug)]
     #[serde(rename_all = "camelCase")]
     pub struct Order {
-        #[serde(flatten)]
-        pub order: super::OrderCreateRequest,
+        pub notify_url: Option<String>,
+        /// Customer client IP address
+        pub customer_ip: String,
+        /// Secret pos ip. This is connected to PayU account
+        #[serde(deserialize_with = "deserialize::deserialize_i32_newtype")]
+        pub merchant_pos_id: super::MerchantPosId,
+        /// Transaction description
+        pub description: String,
+        /// 3 characters currency identifier, ex. PLN
+        pub currency_code: String,
+        /// Total price of the order in pennies (e.g. 1000 is 10.00 EUR).
+        /// Applies also to currencies without subunits (e.g. 1000 is 10
+        /// HUF).
+        #[serde(deserialize_with = "deserialize::deserialize_i32")]
+        pub total_amount: super::Price,
+        /// @see [Buyer]
+        pub buyer: Option<super::Buyer>,
+        /// List of products
+        pub products: Vec<super::Product>,
+        #[serde(skip_serializing)]
+        pub order_create_date: Option<String>,
         pub pay_method: Option<super::PayMethod>,
         pub status: super::PaymentStatus,
     }
-
-    impl std::ops::Deref for Order {
-        type Target = super::OrderCreateRequest;
-
-        fn deref(&self) -> &Self::Target {
-            &self.order
-        }
-    }
-
-    #[derive(Deserialize, Debug)]
-    #[serde(rename_all = "camelCase")]
-    pub struct Prop {
-        pub name: String,
-        pub value: String,
-    }
 }
 
 pub struct Client {
     sandbox: bool,
     merchant_pos_id: MerchantPosId,
-    client_id: String,
-    client_secret: String,
+    client_id: ClientId,
+    client_secret: ClientSecret,
     bearer: Option<String>,
     bearer_expires_at: chrono::DateTime<chrono::Utc>,
     #[cfg(feature = "single-client")]
@@ -667,23 +913,19 @@ pub struct Client {
 
 impl Client {
     /// Create new PayU client
-    pub fn new<ClientId, ClientSecret>(
+    pub fn new(
         client_id: ClientId,
         client_secret: ClientSecret,
         merchant_pos_id: MerchantPosId,
-    ) -> Self
-    where
-        ClientId: Into<String>,
-        ClientSecret: Into<String>,
-    {
+    ) -> Self {
         #[cfg(feature = "single-client")]
         {
             Self {
                 bearer: None,
                 sandbox: false,
                 merchant_pos_id,
-                client_id: client_id.into(),
-                client_secret: client_secret.into(),
+                client_id,
+                client_secret,
                 bearer_expires_at: chrono::Utc::now(),
                 client: Arc::new(Self::build_client()),
             }
@@ -694,8 +936,8 @@ impl Client {
                 bearer: None,
                 sandbox: false,
                 merchant_pos_id,
-                client_id: client_id.into(),
-                client_secret: client_secret.into(),
+                client_id,
+                client_secret,
                 bearer_expires_at: chrono::Utc::now(),
             }
         }
@@ -726,9 +968,9 @@ impl Client {
     /// # Examples
     ///
     /// ```rust
-    /// # use pay_u::{Client, OrderCreateRequest, Product, Buyer};
+    /// # use pay_u::*;
     /// async fn pay() {
-    ///     let mut client = Client::new("145227", "12f071174cb7eb79d4aac5bc2f07563f", 300746)
+    ///     let mut client = Client::new(ClientId::new("145227"), ClientSecret::new("12f071174cb7eb79d4aac5bc2f07563f"), MerchantPosId::new(300746))
     ///         .sandbox()
     ///         .with_bearer("d9a4536e-62ba-4f60-8017-6053211d3f47", 2000);
     ///     let res = client
@@ -748,7 +990,7 @@ impl Client {
     ///             .await;
     /// }
     /// ```
-    pub async fn create_order(&mut self, order: OrderCreateRequest) -> Result<CreateOrderResult> {
+    pub async fn create_order(&mut self, order: OrderCreateRequest) -> Result<res::CreateOrder> {
         self.authorize().await?;
         if order.total_amount
             != order
@@ -779,10 +1021,27 @@ impl Client {
             .text()
             .await?;
         log::trace!("Response: {}", text);
-        serde_json::from_str(&text).map_err(|e| {
+        let res: res::CreateOrder = serde_json::from_str(&text).map_err(|e| {
             log::error!("{e:?}");
             Error::CreateOrder
-        })
+        })?;
+        if !res.status.is_success() {
+            let Status {
+                status_code,
+                status_desc,
+                code,
+                severity,
+                code_literal,
+            } = res.status;
+            return Err(Error::CreateFailed {
+                status_desc,
+                status_code,
+                code,
+                severity,
+                code_literal,
+            });
+        }
+        Ok(res)
     }
 
     /// The PayU system fully supports refunds for processed payments, the
@@ -810,25 +1069,33 @@ impl Client {
     /// ```
     /// # use pay_u::*;
     /// async fn perform_refund() {
-    ///     let mut client = Client::new("145227", "12f071174cb7eb79d4aac5bc2f07563f", 300746)
+    ///     let mut client = Client::new(ClientId::new("145227"), ClientSecret::new("12f071174cb7eb79d4aac5bc2f07563f"), MerchantPosId::new(300746))
     ///         .with_bearer("d9a4536e-62ba-4f60-8017-6053211d3f47", 2000)
     ///         .sandbox();
     ///     let res = client
-    ///         .partial_refund(
-    ///             "H9LL64F37H160126GUEST000P01",
-    ///             RefundRequest::new("Refund", 1000),
+    ///         .refund(
+    ///             OrderId::new("H9LL64F37H160126GUEST000P01"),
+    ///             RefundRequest::new("Refund", Some(1000)),
+    ///         )
+    ///         .await;
+    /// }
+    /// async fn full_refund() {
+    ///     let mut client = Client::new(ClientId::new("145227"), ClientSecret::new("12f071174cb7eb79d4aac5bc2f07563f"), MerchantPosId::new(300746))
+    ///         .with_bearer("d9a4536e-62ba-4f60-8017-6053211d3f47", 2000)
+    ///         .sandbox();
+    ///     let res = client
+    ///         .refund(
+    ///             OrderId::new("H9LL64F37H160126GUEST000P01"),
+    ///             RefundRequest::new("Refund", None),
     ///         )
     ///         .await;
     /// }
     /// ```
-    pub async fn partial_refund<OrderId>(
+    pub async fn refund(
         &mut self,
         order_id: OrderId,
         refund: RefundRequest,
-    ) -> Result<PartialRefundResult>
-    where
-        OrderId: std::fmt::Display,
-    {
+    ) -> Result<res::PartialRefund> {
         self.authorize().await?;
         if refund.description().trim().is_empty() {
             return Err(Error::NoDescription);
@@ -846,9 +1113,129 @@ impl Client {
             .text()
             .await?;
         log::trace!("Response: {}", text);
-        serde_json::from_str::<'_, PartialRefundResult>(&text).map_err(|e| {
+        let res: res::PartialRefund = serde_json::from_str(&text).map_err(|e| {
             log::error!("Invalid PayU response {e:?}");
             Error::Refund
+        })?;
+        if !res.status.is_success() {
+            let Status {
+                status_code,
+                status_desc,
+                code,
+                severity,
+                code_literal,
+            } = res.status;
+            return Err(Error::RefundFailed {
+                status_desc,
+                status_code,
+                code,
+                severity,
+                code_literal,
+            });
+        }
+        Ok(res)
+    }
+
+    /// Order details request. You may use it to completely remove `Order`
+    /// persistence and use extOrderId to connect your data with PayU data.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use pay_u::*;
+    /// async fn order_details() {
+    ///     let mut client = Client::new(ClientId::new("145227"), ClientSecret::new("12f071174cb7eb79d4aac5bc2f07563f"), MerchantPosId::new(300746))
+    ///         .with_bearer("d9a4536e-62ba-4f60-8017-6053211d3f47", 2000)
+    ///         .sandbox();
+    ///     let res = client
+    ///         .order_details(OrderId::new("H9LL64F37H160126GUEST000P01"))
+    ///         .await;
+    /// }
+    /// ```
+    pub async fn order_details(&mut self, order_id: OrderId) -> Result<res::OrderInfo> {
+        self.authorize().await?;
+        let bearer = self.bearer.as_ref().cloned().unwrap_or_default();
+        let path = format!("{}/orders/{}", self.base_url(), order_id);
+        let client = get_client!(self);
+        let text = client
+            .post(path)
+            .bearer_auth(bearer)
+            .send()
+            .await?
+            .text()
+            .await?;
+        log::trace!("Response: {}", text);
+        let mut res: OrdersInfo = serde_json::from_str(&text).map_err(|e| {
+            log::error!("{e:?}");
+            Error::OrderDetails
+        })?;
+        if !res.status.is_success() {
+            let Status {
+                status_code,
+                status_desc,
+                code,
+                severity,
+                code_literal,
+            } = res.status;
+            return Err(Error::OrderDetailsFailed {
+                status_code,
+                status_desc,
+                code,
+                severity,
+                code_literal,
+            });
+        }
+        Ok(res::OrderInfo {
+            order: if res.orders.is_empty() {
+                return Err(Error::NoOrderInDetails);
+            } else {
+                res.orders.remove(0)
+            },
+            status: res.status,
+            properties: res.properties,
+        })
+    }
+
+    /// The transaction retrieve request message enables you to retrieve the
+    /// details of transactions created for an order.
+    ///
+    /// Using this endpoint is extremely useful if you would like to get bank
+    /// account details or card details.
+    ///
+    /// > Please note that although card details are available right after
+    /// > transaction has been processed, the bank details may be available
+    /// > either after few minutes or on the next business day, depending on the
+    /// > bank.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use pay_u::*;
+    /// async fn order_transactions() {
+    ///     let mut client = Client::new(ClientId::new("145227"), ClientSecret::new("12f071174cb7eb79d4aac5bc2f07563f"), MerchantPosId::new(300746))
+    ///         .with_bearer("d9a4536e-62ba-4f60-8017-6053211d3f47", 2000)
+    ///         .sandbox();
+    ///     let res = client
+    ///         .order_transactions(OrderId::new("H9LL64F37H160126GUEST000P01"))
+    ///         .await;
+    /// }
+    /// ```
+    pub async fn order_transactions(&mut self, order_id: OrderId) -> Result<res::Transactions> {
+        self.authorize().await?;
+        let bearer = self.bearer.as_ref().cloned().unwrap_or_default();
+        let path = format!("{}/orders/{}/transactions", self.base_url(), order_id);
+        let client = get_client!(self);
+        let text = client
+            .post(path)
+            .bearer_auth(bearer)
+            .send()
+            .await?
+            .text()
+            .await?;
+        log::trace!("Response: {}", text);
+        serde_json::from_str(&text).map_err(|e| {
+            log::error!("{e:?}");
+            Error::OrderTransactions
         })
     }
 
@@ -908,9 +1295,13 @@ mod tests {
 
     fn build_client() -> Client {
         dotenv::dotenv().ok();
-        Client::new("145227", "12f071174cb7eb79d4aac5bc2f07563f", 300746)
-            .sandbox()
-            .with_bearer("d9a4536e-62ba-4f60-8017-6053211d3f47", 999999)
+        Client::new(
+            ClientId::new("145227"),
+            ClientSecret::new("12f071174cb7eb79d4aac5bc2f07563f"),
+            MerchantPosId::new(300746),
+        )
+        .sandbox()
+        .with_bearer("d9a4536e-62ba-4f60-8017-6053211d3f47", 999999)
     }
 
     #[tokio::test]
@@ -933,27 +1324,69 @@ mod tests {
                 ),
             )
             .await;
+
+        if res.is_err() {
+            eprintln!("create_order res is {res:?}");
+        }
         assert!(res.is_ok());
     }
 
     #[tokio::test]
     async fn partial_refund() {
         let res = build_client()
-            .partial_refund(
-                "H9LL64F37H160126GUEST000P01",
-                RefundRequest::new("Refund", 1000),
+            .refund(
+                OrderId::new("H9LL64F37H160126GUEST000P01"),
+                RefundRequest::new("Refund", Some(1000)),
             )
             .await;
-        eprintln!("{:?}", res);
-        assert!(matches!(res, Ok(_)));
+
+        if res.is_err() {
+            eprintln!("partial_refund res is {res:?}");
+        }
+        assert!(matches!(res, Err(Error::RefundFailed { .. })));
     }
 
     #[tokio::test]
-    async fn check_refund() {}
+    async fn full_refund() {
+        let res = build_client()
+            .refund(
+                OrderId::new("H9LL64F37H160126GUEST000P01"),
+                RefundRequest::new("Refund", None),
+            )
+            .await;
+
+        if res.is_err() {
+            eprintln!("full_refund res is {res:?}");
+        }
+        assert!(matches!(res, Err(Error::RefundFailed { .. })));
+    }
+
+    #[tokio::test]
+    async fn order_details() {
+        let res = build_client()
+            .order_details(OrderId::new("H9LL64F37H160126GUEST000P01"))
+            .await;
+
+        if res.is_err() {
+            eprintln!("order_details res is {res:?}");
+        }
+        assert!(matches!(res, Err(Error::OrderDetails)));
+    }
+
+    #[tokio::test]
+    async fn order_transactions() {
+        let res = build_client()
+            .order_transactions(OrderId::new("H9LL64F37H160126GUEST000P01"))
+            .await;
+        if res.is_err() {
+            eprintln!("order_transactions res is {res:?}");
+        }
+        assert!(matches!(res, Err(Error::OrderTransactions)));
+    }
 
     #[test]
     fn check_accepted_refund_json() {
-        let res = serde_json::from_str::<PartialRefundResult>(include_str!(
+        let res = serde_json::from_str::<res::PartialRefund>(include_str!(
             "../tests/responses/accepted_refund.json"
         ));
         assert!(res.is_ok());
@@ -995,14 +1428,14 @@ mod tests {
     }
     #[test]
     fn check_rejection_json() {
-        let res = serde_json::from_str::<PartialRefundResult>(include_str!(
+        let res = serde_json::from_str::<res::PartialRefund>(include_str!(
             "../tests/responses/rejection.json"
         ));
         assert!(res.is_ok());
     }
     #[test]
     fn check_custom_literal_json() {
-        let res = serde_json::from_str::<PartialRefundResult>(include_str!(
+        let res = serde_json::from_str::<res::PartialRefund>(include_str!(
             "../tests/responses/custom_code_literal.json"
         ));
         assert!(res.is_ok());
